/*jslint indent: 2, white: true, browser: true, devel: true */
/*global navigator,google,Blob,URL */
'use strict';
function App() {
  this.body = null; // html body element
  this.gmap = null; // html div wrapper for google map
  this.stat = null; // html div for status output
  this.btnMark = null; // btml button for start / stop tracking
  this.btnCenter = null; // html button for start / stop center actual position
  this.wakeLock = null; // geolocation wakelock
  this.watchID = null; // geolocation watch id
  this.counter = 0; // geolocation watch position counter
  this.map = null; // google map
  this.marker = null; // google marker
  this.poly = null; // google maps polyline to display track
  this.lastPos = {}; // last position to detect changes
  this.marks = []; // marks cache send to map
  this.track = []; // full track
  this.gps = null; // xml-File
}
App.prototype = {
  // start tracking
  start: function () {
    function formatNum (s, n) {
      s = String(s);
      while (s.length < n) {
        s = "0" + s;
      }
      return s;
    }
    function formatDeg(n) {
      var deg = Math.floor(n), minutes, seconds, cents;
      n = (n - Math.floor(n)) * 60;
      minutes = Math.floor(n);
      n = (n - Math.floor(n)) * 60;
      seconds = Math.floor(n);
      n = (n - Math.floor(n)) * 100;
      cents = Math.floor(n);
      return deg + "°" + formatNum(minutes, 2) + "'" + formatNum(seconds, 2) + "." + formatNum(cents, 2) + '"';
    }
    var self = this;
    this.btnMark.className = "pack-icon-mark active";
    this.stat.innerHTML = 'get it ...';
    this.poly.setMap(this.map);
    this.lastPos = {'lat': 0, 'lon': 0};
    this.wakeLock = navigator.requestWakeLock('gps');
    this.watchID = navigator.geolocation.watchPosition(
      function (pos) {
        var lat, lon, alt, speed, posMark, latlng, path;
        ++self.counter;
        lat = pos.coords.latitude;
        lon = pos.coords.longitude;
        alt = pos.coords.altitude;
        speed = pos.coords.speed;
        if (((self.lastPos.lat !== lat) || (self.lastPos.lon !== lon)) && (pos.coords.accuracy < 32)) {
          self.marks.push({'lat': lat, 'lon': lon});
          self.lastPos = {'lat': lat, 'lon': lon, 'alt': alt, 'speed': speed, 'accuracy': pos.coords.accuracy};
          self.track.push({'lat': lat, 'lon': lon, 'alt': alt, 'ts': pos.timestamp});
        }
        if (!document.hidden && (self.marks.length > 0)) {
          path = self.poly.getPath();
          while (self.marks.length > 0) {
            posMark = self.marks.shift();
            lat = posMark.lat;
            lon = posMark.lon;
            latlng = new google.maps.LatLng(lat, lon);
            path.push(latlng);
          }
          if (latlng) {
            self.marker.setPosition(latlng);
            self.marker.setVisible(true);
            if (self.btnCenter.className === "pack-icon-move active") {
              self.map.panTo(latlng);
            }
          }
        }
        self.stat.innerHTML = "lat: " + ((self.lastPos.lat === null) ? "no lat" : formatDeg(Math.abs(self.lastPos.lat)) + (self.lastPos.lat < 0 ? "S" : "N")) + "<br />" +
                              "lon: " + ((self.lastPos.lon === null) ? "no lon" : formatDeg(Math.abs(self.lastPos.lon)) + (self.lastPos.lon < 0 ? "W" : "E")) + "<br />" +
                              // Referenzelipsoid WGS84 + Quasigeoid height (47.5)
                              "alt: " + ((self.lastPos.alt === null) ? "no alt" : Math.round(self.lastPos.alt - 47.5) + "m NN") + "<br />" +
                              "speed: " + ((self.lastPos.speed !== null && ! isNaN(self.lastPos.speed)) ? (self.lastPos.speed * 3.6).toFixed(0) + "km/h" : "no speed") + "<br />" +
                              "accuracy: ±" + self.lastPos.accuracy + "m" + "<br />" +
                              "count: " + self.track.length + "/"+ self.counter;
      },
      function (error) {
        //self.btnMark.className = "pack-icon-mark";
        ++self.counter;
        self.stat.innerHTML = 'error' + '<br />' + 'try to get it ...';
      },
      {
        enableHighAccuracy: true,
        maximumAge: 3000,
        timeout: 3000
      }
    );
    navigator.vibrate(200);
  },
  // stop tracking
  stop: function () {
    this.btnMark.className = "pack-icon-mark";
    navigator.geolocation.clearWatch(this.watchID);
    this.wakeLock.unlock();
    this.marker.setVisible(false);
  },
  // clear tracking history
  clear: function () {
    var path = this.poly.getPath();
    path.clear();
    this.track = [];
    this.marks = [];
    this.counter = 0;
  },
  // save track
  save: function () {
    var self = this, dateStr = new Date().toISOString(), gpx = [], blob, sdcard, request;
    gpx.push('<?xml version="1.0" encoding="UTF-8" standalone="no" ?>\n');
    gpx.push('<gpx xmlns="http://www.topografix.com/GPX/1/1" version="1.1" creator="www.gocher.me">\n');
    gpx.push('<metadata><link href="http://www.gocher.me"><text>gocher.me</text></link>\n');
    gpx.push('<time>' + dateStr + '</time></metadata>\n');
    gpx.push('<trk>\n');
    gpx.push('  <trkseg>\n');
    this.track.forEach(
      function (pos, i) {
        gpx.push('    <trkpt lat="' + pos.lat + '" lon="' + pos.lon + '">' +
                    ((pos.alt !== undefined) ? ('<ele>' + pos.alt + '</ele>') : '') +
                    '<time>' + new Date(pos.ts).toISOString() + '</time>' +
                 '</trkpt>\n');
      });
    gpx.push('  </trkseg>\n');
    gpx.push('</trk>\n');
    gpx.push('</gpx>\n');
    blob = new Blob(gpx, {'type': 'application/gpx+xml'});
    sdcard = navigator.getDeviceStorage("sdcard");
    request = sdcard.addNamed(blob, 'tracks/' + dateStr.replace('/', '-').replace(':', '-') + '.gpx');
    request.onsuccess = function () {
      var name = this.result;
      alert('File "' + name + '" successfully wrote on the sdcard storage area');
    };
    // An error typically occur if a file with the same name already exist
    request.onerror = function () {
      if (this.error.name === 'SecurityError') {
        // fallback download
        blob.name = dateStr.replace('/', '-').replace(':', '-') + '.gpx';
        self.stat.appendChild(document.createElement('br'));
        var elem = document.createElement('a'),
            gpxUrl = URL.createObjectURL(blob);
        elem.setAttribute('href', gpxUrl);
        elem.setAttribute('download', blob.name);
        self.stat.appendChild(elem);
        elem.appendChild(document.createTextNode('download'));
      } else {
        alert('Unable to write the file: ' + this.error.name);
      }
    };
  },
  // display toolbar
  addToolbar: function () {
    function addButton(className, caption, ul) {
      var li, btn;
      li = document.createElement('li');
      ul.appendChild(li);
      btn = document.createElement('button');
      li.appendChild(btn);
      btn.className = className;
      btn.appendChild(document.createTextNode(caption));
      return btn;
    }
    var self = this, toolbar, ul, btnDelete, btnShare;
    toolbar = document.createElement('div');
    toolbar.setAttribute('role', "toolbar");
    this.body.appendChild(toolbar);
    ul = document.createElement('ul');
    toolbar.appendChild(ul);
    btnDelete = addButton("pack-icon-delete", "Delete", ul);
    btnDelete.onclick = function () {
      self.clear();
    };
    ul = document.createElement('ul');
    toolbar.appendChild(ul);
    this.btnMark = addButton("pack-icon-mark", "Mark", ul);
    this.btnMark.onclick = function () {
      if (self.btnMark.className === "pack-icon-mark") {
        self.start();
      } else {
        self.stop();
      }
    };
    this.btnCenter = addButton("pack-icon-move", "Move", ul);
    this.btnCenter.onclick = function () {
      if (self.btnCenter.className === "pack-icon-move") {
        self.btnCenter.className = "pack-icon-move active";
      } else {
        self.btnCenter.className = "pack-icon-move";
      }
    };
    btnShare = addButton("pack-icon-share", "Share", ul);
    btnShare.onclick = function () {
      self.save();
    };
  },
  // initialize app
  init: function () {
    this.body = document.getElementsByTagName('body')[0];
    this.gmap = document.createElement('div');
    this.gmap.setAttribute('id', 'gmap');
    this.body.appendChild(this.gmap);

    this.stat = document.createElement("div");
    this.stat.setAttribute('id', 'status');
    this.body.appendChild(this.stat);
    this.addToolbar();
    // Google Maps
    this.map = new google.maps.Map(
      this.gmap, {
        zoom: 17,
        zoomControl: false,
        streetViewControl: false,
        scrollwheel: false,
        mapTypeControl: true,
        keyboardShortcuts: false,
        mapMaker: false,
        noClear: true,
        overviewMapControl: false,
        rotateControl: false,
        disableDefaultUI: true,
        center: new google.maps.LatLng(51.528710, 6.289250),
        mapTypeId: google.maps.MapTypeId.TERRAIN
      }
    );
    var symbol = {
      path: 'M0 0 a4 4 0 1 1 0 0.0001 z',
      fillColor: 'red',
      fillOpacity: 0.6,
      scale: 1,
      strokeColor: 'black',
      strokeWeight: 1
    };
    this.marker = new google.maps.Marker({
      position: new google.maps.LatLng(0, 0),
      map: this.map,
      icon: symbol
    });
    this.poly = new google.maps.Polyline({
      strokeColor: '#FF0000',
      strokeOpacity: 0.5,
      strokeWeight: 3
    });
    this.start();
  }
};

window.addEventListener('DOMContentLoaded', function() {
  var app = new App();
  app.init();
});